(PackageInfo named: 'PackageInfo-Base') preamble: '"below, add code to be run before the loading of this package"
PackageOrganizer default
	unregisterPackageNamed: ''PackageInfo'';
	unregisterPackageNamed: ''ToolBuilder'';
	unregisterPackageNamed: ''Morphic-TrueType'''!
SystemOrganization addCategory: #'PackageInfo-Base'!

!Character methodsFor: '*packageinfo-base' stamp: 'ab 5/31/2003 17:15'!
escapeEntities
	#($< '&lt;' $> '&gt;' $& '&amp;') pairsDo:
		[:k :v |
		self = k ifTrue: [^ v]].
	^ String with: self! !

!Collection methodsFor: '*packageinfo-base' stamp: 'ab 9/30/2002 19:26'!
gather: aBlock
	^ Array streamContents:
		[:stream |
		self do: [:ea | stream nextPutAll: (aBlock value: ea)]]! !

Object subclass: #PackageInfo
	instanceVariableNames: 'packageName methodCategoryPrefix preamble postscript preambleOfRemoval postscriptOfRemoval resources window'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

PackageInfo class
	instanceVariableNames: 'default'!

!PackageInfo commentStamp: '<historical>' prior: 0!
Subclass this class to create new Packages.!

PackageInfo class
	instanceVariableNames: 'default'!

!PackageInfo class methodsFor: 'packages access' stamp: 'nk 3/9/2004 10:49'!
allPackages
	^PackageOrganizer default packages! !

!PackageInfo class methodsFor: 'compatibility' stamp: 'avi 3/9/2004 16:28'!
default
	^ self allPackages detect: [:ea | ea class = self] ifNone: [self new register]! !

!PackageInfo class methodsFor: 'class initialization' stamp: 'avi 2/18/2004 00:46'!
initialize
	self allSubclassesDo: [:ea | ea new register]! !

!PackageInfo class methodsFor: 'packages access' stamp: 'avi 11/12/2003 23:00'!
named: aString
	^ PackageOrganizer default packageNamed: aString ifAbsent: [(self new packageName: aString) register]! !

!PackageInfo class methodsFor: 'registration / unregistration' stamp: 'ab 11/14/2002 15:05'!
registerPackage: aString
	"for compatibility with old fileOuts"
	^ Smalltalk at: #FilePackageManager ifPresent: [:p | p registerPackage: aString]! !

!PackageInfo class methodsFor: 'registration / unregistration' stamp: 'avi 11/11/2003 17:19'!
registerPackageName: aString
	^ PackageOrganizer default registerPackageNamed: aString! !

!PackageInfo methodsFor: 'comparing' stamp: 'avi 10/11/2003 00:09'!
= other
	^ other species = self species and: [other packageName = self packageName]! !

!PackageInfo methodsFor: 'enumerating' stamp: 'eem 8/30/2010 10:28'!
actualMethodsDo: aBlock
	"Evaluate aBlock with the actual method objects in this package."
	| enum |
	self extensionMethods do:
		[:mr|
		aBlock value: mr compiledMethod].
	enum := [:behavior|
			behavior organization categories do:
				[:cat|
				(self isForeignClassExtension: cat) ifFalse:
					[(behavior organization listAtCategoryNamed: cat) do:
						[:s|
						aBlock value: (behavior compiledMethodAt: s)]]]].
	self classes do:
		[:c| enum value: c; value: c classSide]
		! !

!PackageInfo methodsFor: 'modifying' stamp: 'avi 10/13/2003 15:40'!
addCoreMethod: aMethodReference
	| category |
	category := self baseCategoryOfMethod: aMethodReference.
	aMethodReference actualClass organization
		classify: aMethodReference methodSymbol
		under: category
		suppressIfDefault: false! !

!PackageInfo methodsFor: 'modifying' stamp: 'avi 10/11/2003 15:17'!
addExtensionMethod: aMethodReference
	| category |
	category := self baseCategoryOfMethod: aMethodReference.
	aMethodReference actualClass organization
		classify: aMethodReference methodSymbol
		under: self methodCategoryPrefix, '-', category! !

!PackageInfo methodsFor: 'modifying' stamp: 'avi 10/11/2003 15:16'!
addMethod: aMethodReference
	(self includesClass: aMethodReference class)
		ifTrue: [self addCoreMethod: aMethodReference]
		ifFalse: [self addExtensionMethod: aMethodReference]! !

!PackageInfo methodsFor: 'resources' stamp: 'edc 3/3/2013 07:44'!
addResources
	resources
		ifNil: [resources := Dictionary new].
	window := PackageResourcesList new.
	window thisPackage: self.
	window initialize.
	window openInWorld! !

!PackageInfo methodsFor: 'listing' stamp: 'bp 2/26/2010 20:45'!
allOverriddenMethods
	"search classes and meta classes"
	^ Array streamContents: [:stream |
		self allOverriddenMethodsDo: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'bp 2/26/2010 20:47'!
allOverriddenMethodsDo: aBlock
	"Evaluates aBlock with all the overridden methods in the system"
	^ ProtoObject withAllSubclassesDo: [:class | 
		self overriddenMethodsInClass: class do: aBlock]
! !

!PackageInfo methodsFor: 'modifying' stamp: 'avi 10/13/2003 15:39'!
baseCategoryOfMethod: aMethodReference
	| oldCat oldPrefix tokens | 
	oldCat := aMethodReference category.
	({ 'as yet unclassified'. 'all' } includes: oldCat) ifTrue: [ oldCat := '' ].
	tokens := oldCat findTokens: '*-' keep: '*'.

	"Strip off any old prefixes"
	((tokens at: 1 ifAbsent: [ '' ]) = '*') ifTrue: [
		[ ((tokens at: 1 ifAbsent: [ '' ]) = '*') ]
			whileTrue: [ tokens removeFirst ].
		oldPrefix := tokens removeFirst asLowercase.
		[ (tokens at: 1 ifAbsent: [ '' ]) asLowercase = oldPrefix ]
			whileTrue: [ tokens removeFirst ].
	].

	tokens isEmpty ifTrue: [^ 'as yet unclassified'].
	^ String streamContents:
		[ :s |
		tokens
			do: [ :tok | s nextPutAll: tok ]
			separatedBy: [ s nextPut: $- ]]! !

!PackageInfo methodsFor: 'testing' stamp: 'nk 4/28/2004 14:20'!
category: categoryName matches: prefix
	| prefixSize catSize |
	categoryName ifNil: [ ^false ].
	catSize := categoryName size.
	prefixSize := prefix size.
	catSize < prefixSize ifTrue: [ ^false ].
	(categoryName findString: prefix startingAt: 1 caseSensitive: false) = 1
		ifFalse: [ ^false ].
	^(categoryName at: prefix size + 1 ifAbsent: [ ^true ]) = $-! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 10/17/2002 00:05'!
categoryName
	|category|
	category := self class category.
	^ (category endsWith: '-Info')
		ifTrue: [category copyUpToLast: $-]
		ifFalse: [category]! !

!PackageInfo methodsFor: 'testing' stamp: 'bf 5/4/2012 15:48'!
changeRecordForOverriddenMethod: aMethodReference
	self changeRecordsForMethod: aMethodReference do: [:record |
		(self includesMethodCategory: record category
			ofClass: aMethodReference actualClass)
				ifTrue: [^record]].
	^nil! !

!PackageInfo methodsFor: 'enumerating' stamp: 'bf 5/4/2012 15:50'!
changeRecordsForMethod: aMethodReference do: aBlock
	"Evaluate aBlock with one ChangeRecord per overriding package, followed by the latest non-override"
	| overridingPackages method position sourceFilesCopy |
	overridingPackages := Set new.
	method := aMethodReference compiledMethod.
	position := method filePosition.
	sourceFilesCopy := SourceFiles collect: [:x | x ifNotNil: [x readOnlyCopy]].
	[ | file prevPos prevFileIndex chunk stamp methodCategory methodPackage tokens |
	method fileIndex = 0 ifTrue: [^ nil].
	file := sourceFilesCopy at: method fileIndex.
	[position notNil & file notNil]
		whileTrue:
		[file position: (0 max: position-150).  "Skip back to before the preamble"
		[file position < (position-1)]  "then pick it up from the front"
			whileTrue: [chunk := file nextChunk].

		"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
		prevPos := nil.
		stamp := ''.
		(chunk findString: 'methodsFor:' startingAt: 1) > 0
			ifTrue: [tokens := Scanner new scanTokens: chunk]
			ifFalse: [tokens := Array new  "ie cant be back ref"].
		((tokens size between: 7 and: 8)
			and: [(tokens at: tokens size-5) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-3) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size-2.
						prevPos := tokens last.
						prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
						prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos]
				ifFalse: ["Old format gives no stamp; prior pointer in two parts"
						prevPos := tokens at: tokens size-2.
						prevFileIndex := tokens last].
				(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos := nil]].
		((tokens size between: 5 and: 6)
			and: [(tokens at: tokens size-3) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-1) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size]].
		methodCategory := tokens after: #methodsFor: ifAbsent: [ClassOrganizer default].
		methodPackage := PackageOrganizer default packageOfMethodCategory: methodCategory ofClass: aMethodReference actualClass ifNone: [#unknown].
		(overridingPackages includes: methodPackage)
			ifFalse: [aBlock value: (ChangeRecord new
				file: file position: position type: #method
				class: aMethodReference classSymbol category: methodCategory
				meta: aMethodReference classIsMeta stamp: stamp)].
		(self isOverrideCategory: methodCategory)
			ifTrue: [overridingPackages add: methodPackage]
			ifFalse: [(overridingPackages includes: methodPackage)
				ifFalse: [^nil]].
		position := prevPos.
		prevPos notNil ifTrue:
			[file := sourceFilesCopy at: prevFileIndex]].
		^nil]
			ensure: [sourceFilesCopy do: [:x | x ifNotNil: [x close]]]
	! !

!PackageInfo methodsFor: 'listing' stamp: 'ac 5/14/2003 16:23'!
classes
	^(self systemCategories gather:
		[:cat |
		(SystemOrganization listAtCategoryNamed: cat)
			collect: [:className | Smalltalk at: className]])
				sortBy: [:a :b | a className <= b className]! !

!PackageInfo methodsFor: 'listing' stamp: 'ul 11/13/2009 16:29'!
classesAndMetaClasses
	"Return a Set with all classes and metaclasses belonging to this package"

	| baseClasses result |
	baseClasses := self classes.
	result := (Set new: baseClasses size * 2) 
		addAll: baseClasses;
		yourself.
	baseClasses do: [ :c | 
		result add: c classSide].
	^result
! !

!PackageInfo methodsFor: 'resources' stamp: 'edc 6/2/2009 08:40'!
clearResources
	 resources :=Dictionary new! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:18'!
coreCategoriesForClass: aClass
	^ aClass organization categories select: [:cat | (self isForeignClassExtension: cat) not]! !

!PackageInfo methodsFor: 'listing' stamp: 'ab 11/13/2002 01:23'!
coreMethods
	^ self classesAndMetaClasses gather: [:class | self coreMethodsForClass: class]! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:22'!
coreMethodsForClass: aClass
	^ (aClass selectors difference:
		((self foreignExtensionMethodsForClass: aClass) collect: [:r | r methodSymbol]))
			asArray collect: [:sel | self referenceForMethod: sel ofClass: aClass]! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:20'!
extensionCategoriesForClass: aClass
	^ aClass organization categories select: [:cat | self isYourClassExtension: cat]! !

!PackageInfo methodsFor: 'listing' stamp: 'al 3/1/2006 21:51'!
extensionClasses
	^ self externalBehaviors reject: [:classOrTrait | (self extensionCategoriesForClass: classOrTrait) isEmpty]! !

!PackageInfo methodsFor: 'listing' stamp: 'al 3/1/2006 21:51'!
extensionMethods
	^ self externalBehaviors gather: [:classOrTrait | self extensionMethodsForClass: classOrTrait]! !

!PackageInfo methodsFor: 'testing' stamp: 'avi 9/10/2004 18:28'!
extensionMethodsForClass: aClass
	^ (self extensionCategoriesForClass: aClass)
		gather: [:cat | self methodsInCategory: cat ofClass: aClass ]! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 10/18/2002 23:22'!
extensionMethodsFromClasses: classes
	^classes
		gather: [:class | self extensionMethodsForClass: class]! !

!PackageInfo methodsFor: 'modifying' stamp: 'al 3/1/2006 21:42'!
externalBehaviors
	^self externalClasses , self externalTraits! !

!PackageInfo methodsFor: 'dependencies' stamp: 'ab 11/18/2002 01:16'!
externalCallers
	^ self 
		externalRefsSelect: [:literal | literal isKindOf: Symbol] 
		thenCollect: [:l | l].! !

!PackageInfo methodsFor: 'dependencies' stamp: 'ab 6/10/2003 17:18'!
externalClasses
	| myClasses |
	myClasses := self classesAndMetaClasses.
	^ Array streamContents:
		[:s |
		ProtoObject withAllSubclassesDo:
			[:class |
			(myClasses includes: class) ifFalse: [s nextPut: class]]]! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 10/16/2002 21:22'!
externalName
	^ self packageName! !

!PackageInfo methodsFor: 'dependencies' stamp: 'edc 5/31/2008 09:43'!
externalPackages
	 ^self externalUsers collect: [:mr | mr key systemPackage]
		! !

!PackageInfo methodsFor: 'dependencies' stamp: 'nice 12/26/2009 19:07'!
externalRefsSelect: selBlock thenCollect: colBlock
	| pkgMethods dependents extMethods otherClasses otherMethods classNames |

	classNames := self classes collect: [:c | c name].
	extMethods := self extensionMethods collect: [:mr | mr methodSymbol].
	otherClasses := self externalClasses difference: self externalSubclasses.
	otherMethods :=  otherClasses gather: [:c | c selectors].
	pkgMethods := self methods asSet collect: [:mr | mr methodSymbol].
	pkgMethods removeAllFoundIn: otherMethods.

	dependents := Set new.
	otherClasses do: [:c |
		c selectorsAndMethodsDo:
			[:sel :compiled |
			| refs |
			(extMethods includes: sel) ifFalse: 
				[refs := compiled literals select: selBlock thenCollect: colBlock.
				refs do: [:ea |
					((classNames includes: ea) or: [pkgMethods includes: ea])
							ifTrue: [dependents add: (self referenceForMethod: sel ofClass: c) -> ea]]]]].
	^ dependents! !

!PackageInfo methodsFor: 'dependencies' stamp: 'cwp 11/13/2002 00:24'!
externalSubclasses
	| pkgClasses subClasses |
	pkgClasses := self classes.
	subClasses := Set new.
	pkgClasses do: [:c | subClasses addAll: (c allSubclasses)].
	^ subClasses difference: pkgClasses
! !

!PackageInfo methodsFor: 'modifying' stamp: 'nice 12/26/2009 19:07'!
externalTraits
	^ Array streamContents: [:s |
		| behaviors |
		behaviors := self classesAndMetaClasses.
		Smalltalk allTraits do: [:trait |
			(behaviors includes: trait) ifFalse: [s nextPut: trait].
			(behaviors includes: trait classSide) ifFalse: [s nextPut: trait classSide]]].			! !

!PackageInfo methodsFor: 'dependencies' stamp: 'ab 11/18/2002 01:15'!
externalUsers
	^ self 
		externalRefsSelect: [:literal | literal isVariableBinding] 
		thenCollect: [:l | l key]! !

!PackageInfo methodsFor: 'listing' stamp: 'ab 12/3/2002 14:38'!
foreignClasses
	| s |
	s := IdentitySet new.
	self foreignSystemCategories
		do: [:c | (SystemOrganization listAtCategoryNamed: c)
				do: [:cl | 
					| cls | 
					cls := Smalltalk at: cl. 
					s add: cls;
					  add: cls class]].
	^ s! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:22'!
foreignExtensionCategoriesForClass: aClass
	^ aClass organization categories select: [:cat | self isForeignClassExtension: cat]! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:23'!
foreignExtensionMethodsForClass: aClass
	^ (self foreignExtensionCategoriesForClass: aClass)
		gather: [:cat | (aClass organization listAtCategoryNamed: cat)
						  collect: [:sel | self referenceForMethod: sel ofClass: aClass]]! !

!PackageInfo methodsFor: 'listing' stamp: 'ab 12/3/2002 14:34'!
foreignSystemCategories
	^ SystemOrganization categories
		reject: [:cat | self includesSystemCategory: cat] ! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:57'!
hasPostscript
	^ self isScript: postscript not: self postscriptDefault! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 18:00'!
hasPostscriptOfRemoval
	^ self isScript: postscriptOfRemoval not: self postscriptOfRemovalDefault! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:59'!
hasPreamble
	^ self isScript: preamble not: self preambleDefault! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 18:00'!
hasPreambleOfRemoval
	^ self isScript: preambleOfRemoval not: self preambleOfRemovalDefault! !

!PackageInfo methodsFor: 'testing' stamp: 'edc 12/2/2006 10:05'!
hasResources
^ resources ~= nil! !

!PackageInfo methodsFor: 'comparing' stamp: 'avi 10/11/2003 14:20'!
hash
	^ packageName hash! !

!PackageInfo methodsFor: 'testing' stamp: 'avi 9/14/2004 13:34'!
includesChangeRecord: aChangeRecord
	^ aChangeRecord methodClass notNil and:
		[self
			includesMethodCategory: aChangeRecord category
			ofClass: aChangeRecord methodClass]! !

!PackageInfo methodsFor: 'testing' stamp: 'nice 2/15/2011 23:36'!
includesClass: aClass
	^ self includesSystemCategory: aClass category! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 7/23/2003 14:08'!
includesClassNamed: aClassName
	^ self includesSystemCategory: ((SystemOrganization categoryOfElement: aClassName) ifNil: [^false])! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 12/5/2002 00:16'!
includesMethod: aSymbol ofClass: aClass
	aClass ifNil: [^ false].
	^ self
		includesMethodCategory: ((aClass organization categoryOfElement: aSymbol)
										ifNil: [' '])
		ofClass: aClass! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 9/17/2002 00:18'!
includesMethodCategory: categoryName ofClass: aClass
	^ (self isYourClassExtension: categoryName)
		or: [(self includesClass: aClass)
				and: [(self isForeignClassExtension: categoryName) not]]! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 7/23/2003 14:06'!
includesMethodCategory: categoryName ofClassNamed: aClass
	^ (self isYourClassExtension: categoryName)
		or: [(self includesClassNamed: aClass)
				and: [(self isForeignClassExtension: categoryName) not]]! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/14/2002 18:06'!
includesMethodReference: aMethodRef
	^ self includesMethod: aMethodRef methodSymbol ofClass: aMethodRef actualClass! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:23'!
includesSystemCategory: categoryName
	^ self category: categoryName matches: self systemCategoryPrefix! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:23'!
isForeignClassExtension: categoryName
	^ categoryName first = $* and: [(self isYourClassExtension: categoryName) not]! !

!PackageInfo methodsFor: 'testing' stamp: 'bf 5/4/2012 15:18'!
isOverrideCategory: aString
	^ aString first = $* and: [aString endsWith: '-override']! !

!PackageInfo methodsFor: 'testing' stamp: 'avi 9/10/2004 18:27'!
isOverrideMethod: aMethodReference
	^ self isOverrideCategory: aMethodReference category! !

!PackageInfo methodsFor: 'testing' stamp: 'ar 12/23/2009 18:32'!
isOverrideOfYourMethod: aMethodReference
	"Answers true if the argument overrides a method in this package"
	^ (self isYourClassExtension: aMethodReference category) not and:
		[(self changeRecordForOverriddenMethod: aMethodReference) notNil]! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 18:07'!
isScript: script not: default
	^ script notNil
		and: [ | contents |
			contents := script contents asString withBlanksTrimmed.
			contents notEmpty and: [contents ~= default and: [contents ~= 'nil']]]! !

!PackageInfo methodsFor: 'testing' stamp: 'avi 3/10/2004 12:37'!
isYourClassExtension: categoryName
	^ categoryName notNil and: [self category: categoryName asLowercase matches: self methodCategoryPrefix]! !

!PackageInfo methodsFor: 'source code management' stamp: 'bp 2/26/2010 21:14'!
linesOfCode
	"An approximate measure of lines of code.
	Includes comments, but excludes blank lines."
	^self methods inject: 0 into: [:sum :each | sum + each compiledMethod linesOfCode]! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 6/10/2003 17:21'!
methodCategoryPrefix
	^ methodCategoryPrefix ifNil: [methodCategoryPrefix := '*', self packageName asLowercase]! !

!PackageInfo methodsFor: 'listing' stamp: 'nice 1/1/2010 21:54'!
methods
	^ (self extensionMethods, self coreMethods) select: [:method |
		method isValid
			and: [method isLocalSelector
			and: [method methodSymbol isDoIt not]]]! !

!PackageInfo methodsFor: 'testing' stamp: 'bf 5/4/2005 16:41'!
methodsInCategory: aString ofClass: aClass 
	^Array streamContents: [:stream |
		self methodsInCategory: aString ofClass: aClass 
			do: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'bf 5/4/2005 16:29'!
methodsInCategory: aString ofClass: aClass do: aBlock
	((aClass organization listAtCategoryNamed: aString) ifNil: [^self])
		do: [:sel | aBlock value: (self referenceForMethod: sel ofClass: aClass)]! !

!PackageInfo methodsFor: 'resources' stamp: 'edc 6/2/2009 10:30'!
moreResources
	| result full |
	result := FileList2 modalFileSelector.
	full := result directory fullNameFor: result name.
	resources
		at: result localName
		put: (RWBinaryOrTextStream with: (StandardFileStream readOnlyFileNamed: full) contents).
	^self viewResources! !

!PackageInfo methodsFor: 'naming' stamp: 'mist 2/17/2005 12:55'!
name

^ self packageName! !

!PackageInfo methodsFor: 'testing' stamp: 'nice 1/5/2010 14:01'!
outsideClasses
	^ProtoObject withAllSubclasses asSet difference: self classesAndMetaClasses! !

!PackageInfo methodsFor: 'listing' stamp: 'bf 5/4/2005 17:40'!
overriddenMethods
	^ Array streamContents: [:stream |
		self overriddenMethodsDo: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'bp 2/26/2010 20:46'!
overriddenMethodsDo: aBlock
	"Enumerates the methods the receiver contains which have been overridden by other packages"
	^ self allOverriddenMethodsDo: [:ea |
		(self isOverrideOfYourMethod: ea)
			ifTrue: [aBlock value: ea]]! !

!PackageInfo methodsFor: 'listing' stamp: 'bf 5/4/2005 16:43'!
overriddenMethodsInClass: aClass
	^Array streamContents: [:stream |
		self overriddenMethodsInClass: aClass
			do: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'ar 12/23/2009 18:35'!
overriddenMethodsInClass: aClass do: aBlock
	"Evaluates aBlock with the overridden methods in aClass"
	^ self overrideCategoriesForClass: aClass do: [:cat |
		self methodsInCategory: cat ofClass: aClass do: aBlock]! !

!PackageInfo methodsFor: 'testing' stamp: 'bf 5/4/2005 16:44'!
overrideCategoriesForClass: aClass
	^Array streamContents: [:stream |
		self overrideCategoriesForClass: aClass
			do: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'ar 12/23/2009 18:28'!
overrideCategoriesForClass: aClass do: aBlock
	"Evaluates aBlock with all the *foo-override categories in aClass"
	^ aClass organization categories do: [:cat |
		(self isOverrideCategory: cat) ifTrue: [aBlock value: cat]]! !

!PackageInfo methodsFor: 'listing' stamp: 'nk 5/1/2004 08:40'!
overrideMethods
	^ self extensionMethods select: [:ea | self isOverrideMethod: ea]! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 10/16/2002 16:57'!
packageName
	^ packageName ifNil: [packageName := self categoryName]! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 10/16/2002 16:56'!
packageName: aString
	packageName := aString! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:51'!
postscript
	^ postscript ifNil: [
		postscript := StringHolder new contents: self postscriptDefault]! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'mist 2/18/2005 22:44'!
postscript: aString

postscript := StringHolder new contents: aString! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:49'!
postscriptDefault
	^ '"below, add code to be run after the loading of this package"'! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:51'!
postscriptOfRemoval
	^ postscriptOfRemoval ifNil: [
		postscriptOfRemoval := StringHolder new contents: self postscriptOfRemovalDefault]! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'mist 2/18/2005 22:45'!
postscriptOfRemoval: aString

postscriptOfRemoval := StringHolder new contents: aString
! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:50'!
postscriptOfRemovalDefault
	^ '"below, add code to clean up after the unloading of this package"'! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:51'!
preamble
	^ preamble ifNil: [
		preamble := StringHolder new contents: self preambleDefault]! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'mist 2/19/2005 00:37'!
preamble: aString

preamble := StringHolder new contents: aString! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:49'!
preambleDefault
	^ '"below, add code to be run before the loading of this package"'
! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:49'!
preambleOfRemoval
	^ preambleOfRemoval ifNil: [
		preambleOfRemoval := StringHolder new contents: self preambleOfRemovalDefault]! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'mist 2/19/2005 00:41'!
preambleOfRemoval: aString

preambleOfRemoval := StringHolder new contents: aString
! !

!PackageInfo methodsFor: 'preamble/postscript' stamp: 'bf 5/31/2012 17:47'!
preambleOfRemovalDefault
	^'"below, add code to prepare for the unloading of this package"'! !

!PackageInfo methodsFor: 'printing' stamp: 'bp 2/26/2010 20:37'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self packageName;
		nextPut: $)! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:25'!
referenceForMethod: aSymbol ofClass: aClass
	^ MethodReference new setStandardClass: aClass methodSymbol: aSymbol! !

!PackageInfo methodsFor: 'registering' stamp: 'avi 11/12/2003 23:12'!
register
	PackageOrganizer default registerPackage: self! !

!PackageInfo methodsFor: 'modifying' stamp: 'avi 10/11/2003 15:14'!
removeMethod: aMethodReference! !

!PackageInfo methodsFor: 'resources' stamp: 'edc 6/2/2009 10:33'!
resources
	
	^ resources! !

!PackageInfo methodsFor: 'resources' stamp: 'edc 10/26/2011 09:28'!
resources: anObject
	"Set the value of resources"

	resources := anObject! !

!PackageInfo methodsFor: 'listing' stamp: 'ab 11/14/2002 18:39'!
selectors
	^ self methods collect: [:ea | ea methodSymbol]! !

!PackageInfo methodsFor: 'listing' stamp: 'ab 11/11/2002 21:51'!
systemCategories
	^ SystemOrganization categories select: [:cat | self includesSystemCategory: cat]! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 10/28/2002 10:38'!
systemCategoryPrefix
	^ self packageName! !

!PackageInfo methodsFor: 'resources' stamp: 'edc 6/2/2009 10:29'!
viewResources
	"Answer the value of resources"
	resources
		ifEmpty: [^#()]
		ifNotEmpty: [^ resources keys asSortedCollection]! !

Object subclass: #PackageList
	instanceVariableNames: 'selectedPackage packages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

!PackageList class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 14:43'!
initialize
	TheWorldMenu registerOpenCommand: {'Package List'. {self. #open}}! !

!PackageList class methodsFor: 'as yet unclassified' stamp: 'avi 10/10/2003 22:38'!
open
	^ self new openInWorld! !

!PackageList methodsFor: 'actions' stamp: 'rbb 3/1/2005 11:05'!
addPackage
	| packageName |
	packageName := UIManager default request: 'Package name:'.
	packageName isEmpty ifFalse:
		[selectedPackage := self packageOrganizer registerPackageNamed: packageName.
		self changed: #packageSelection]! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/10/2003 22:37'!
buildList
	^ PluggableListMorph
		on: self
		list: #packageList
		selected: #packageSelection
		changeSelected: #packageSelection:
		menu: #packageMenu:! !

!PackageList methodsFor: 'morphic' stamp: 'avi 2/18/2004 00:28'!
buildWindow
	| window |
	window := SystemWindow labelled: self label.
	window model: self.
	window addMorph: self buildList fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1)).
	^ window! !

!PackageList methodsFor: 'morphic' stamp: 'avi 2/18/2004 00:28'!
defaultBackgroundColor 
	^ Color white! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/11/2003 00:28'!
defaultExtent
	^ 200@200! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/10/2003 22:36'!
label
	^ 'Packages'! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/11/2003 00:24'!
openInWorld
	self packageOrganizer addDependent: self.
	self buildWindow openInWorldExtent: self defaultExtent! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/11/2003 13:09'!
packageContextMenu: aMenu
	aMenu
		addLine;
		add: 'remove package' action: #removePackage;
		addServices: PackageServices allServices for: selectedPackage extraLines: #()! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/11/2003 00:10'!
packageList
	^ self packages collect: [:ea | ea packageName]! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/11/2003 00:24'!
packageMenu: aMenu
	aMenu
		defaultTarget: self;
		add: 'add package' action: #addPackage.
	selectedPackage ifNotNil: [self packageContextMenu: aMenu].
	^ aMenu! !

!PackageList methodsFor: 'actions' stamp: 'avi 10/11/2003 00:17'!
packageOrganizer
	^ PackageOrganizer default! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/10/2003 22:41'!
packageSelection
	^ self packages indexOf: selectedPackage! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/10/2003 22:41'!
packageSelection: aNumber
	selectedPackage := self packages at: aNumber ifAbsent: [].
	self changed: #packageSelection! !

!PackageList methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 00:18'!
packages
	^ packages ifNil: [packages := self packageOrganizer packages asSortedCollection:
									[:a :b | a packageName <= b packageName]]! !

!PackageList methodsFor: 'morphic' stamp: 'avi 10/11/2003 00:15'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!PackageList methodsFor: 'actions' stamp: 'avi 10/11/2003 00:24'!
removePackage
	self packageOrganizer unregisterPackage: selectedPackage! !

!PackageList methodsFor: 'actions' stamp: 'avi 10/11/2003 00:23'!
update: aSymbol
	aSymbol = #packages ifTrue:
		[packages := nil.
		self changed: #packageList; changed: #packageSelection]! !

Object subclass: #PackageOrganizer
	instanceVariableNames: 'packages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

PackageOrganizer class
	instanceVariableNames: 'default'!

PackageOrganizer class
	instanceVariableNames: 'default'!

!PackageOrganizer class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 00:17'!
default
	^ default ifNil: [default := self new]! !

!PackageOrganizer class methodsFor: 'as yet unclassified' stamp: 'avi 10/13/2003 15:25'!
new
	^ self basicNew initialize! !

!PackageOrganizer methodsFor: 'registering' stamp: 'ar 6/16/2010 19:57'!
flushObsoletePackages: aBlock
	"Flush all packages considered obsolete by evaluating the argument block."

	packages keys do:[:key|
		(aBlock value: (packages at: key)) ifTrue:[packages removeKey: key].
	].
	self changed: #packages; changed: #packageNames.! !

!PackageOrganizer methodsFor: 'initializing' stamp: 'avi 11/12/2003 23:01'!
initialize
	packages := Dictionary new! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:21'!
noPackageFound
	self error: 'No package found'! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 11/12/2003 23:08'!
packageNamed: aString ifAbsent: errorBlock
	^ packages at: aString ifAbsent: errorBlock! !

!PackageOrganizer methodsFor: 'accessing' stamp: 'avi 11/12/2003 23:01'!
packageNames
	^ packages keys! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:21'!
packageOfClass: aClass
	^ self packageOfClass: aClass ifNone: [self noPackageFound]! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:22'!
packageOfClass: aClass ifNone: errorBlock
	^ self packages detect: [:ea | ea includesClass: aClass] ifNone: errorBlock! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:21'!
packageOfMethod: aMethodReference
	^ self packageOfMethod: aMethodReference ifNone: [self noPackageFound]! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:22'!
packageOfMethod: aMethodReference ifNone: errorBlock
	^ self packages detect: [:ea | ea includesMethodReference: aMethodReference] ifNone: errorBlock! !

!PackageOrganizer methodsFor: 'searching' stamp: 'nk 6/24/2004 16:02'!
packageOfMethodCategory: categoryName ofClass: aClass
	^self packageOfMethodCategory: categoryName ofClass: aClass ifNone: [ self noPackageFound ]
! !

!PackageOrganizer methodsFor: 'searching' stamp: 'bf 4/19/2012 18:09'!
packageOfMethodCategory: categoryName ofClass: aClass ifNone: errorBlock
	^ self packages detect: [:ea | ea includesMethodCategory: categoryName ofClassNamed: aClass name] ifNone: errorBlock
	
	
! !

!PackageOrganizer methodsFor: 'searching' stamp: 'nk 6/24/2004 15:55'!
packageOfSystemCategory: aSystemCategory
	^ self packageOfSystemCategory: aSystemCategory ifNone: [ self noPackageFound ]
! !

!PackageOrganizer methodsFor: 'searching' stamp: 'nk 6/24/2004 15:54'!
packageOfSystemCategory: aSystemCategory ifNone: errorBlock
	^ self packages detect: [:ea | ea includesSystemCategory: aSystemCategory] ifNone: errorBlock
! !

!PackageOrganizer methodsFor: 'accessing' stamp: 'avi 11/12/2003 23:01'!
packages
	^ packages values! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 23:01'!
registerPackage: aPackageInfo
	packages at: aPackageInfo packageName put: aPackageInfo.
	self changed: #packages; changed: #packageNames.
! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 21:08'!
registerPackageNamed: aString
	^ self registerPackage: (PackageInfo named: aString)! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 23:08'!
unregisterPackage: aPackageInfo
	packages removeKey: aPackageInfo packageName ifAbsent: [].	
	self changed: #packages; changed: #packageNames.
! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 21:10'!
unregisterPackageNamed: aString
	self unregisterPackage: (self packageNamed: aString ifAbsent: [^ self])! !

Object subclass: #PackageResourcesList
	instanceVariableNames: 'resources selectedResource thisPackage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

!PackageResourcesList class methodsFor: 'as yet unclassified' stamp: 'edc 1/26/2013 06:25'!
open
	^ self new openInWorld! !

!PackageResourcesList class methodsFor: 'as yet unclassified' stamp: 'edc 1/26/2013 06:41'!
openOn: aPackageInfo
| instance |
instance := self new.
instance thisPackage: aPackageInfo .
	^  instance openInWorld! !

!PackageResourcesList methodsFor: 'actions' stamp: 'edc 1/26/2013 07:47'!
addResource
	| result full |
	result := FileList2 modalFileSelector.
	full := result directory fullNameFor: result name.
	thisPackage resources
		at: result localName
		put: (RWBinaryOrTextStream with: (StandardFileStream readOnlyFileNamed: full) contents).
		selectedResource := thisPackage resources at: result localName.
		self changed: #packageSelection.
		self changed:  #resources! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 07:04'!
buildList
	^ PluggableListMorph
		on: self
		list: #resources
		selected: #resourceSelection
		changeSelected: #resourceSelection:
		menu: #resourceMenu:! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 06:25'!
buildWindow
	| window |
	window := SystemWindow labelled: self label.
	window model: self.
	window addMorph: self buildList fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1)).
	^ window! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 06:25'!
defaultBackgroundColor 
	^ Color white! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 06:25'!
defaultExtent
	^ 200@200! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 07:12'!
label
	^ 'PackageResourcesList'! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 08:20'!
openInWorld
	
	self buildWindow openInWorldExtent: self defaultExtent! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 06:25'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!PackageResourcesList methodsFor: 'actions' stamp: 'edc 1/26/2013 06:25'!
removePackage
	self packageOrganizer unregisterPackage: selectedPackage! !

!PackageResourcesList methodsFor: 'actions' stamp: 'edc 1/26/2013 08:22'!
removeResource

	thisPackage resources
		removeKey: selectedResource.
		self changed: #packageSelection.
		self changed:  #resources! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 08:14'!
resourceContextMenu: aMenu
	aMenu
		addLine;
		add: 'remove resource' action: #removeResource;
		addServices: PackageServices allServices for: selectedResource extraLines: #()! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 07:26'!
resourceMenu: aMenu
	aMenu
		defaultTarget: self;
		add: 'add resource' action: #addResource.
	selectedResource ifNotNil: [self resourceContextMenu: aMenu].
	^ aMenu! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 07:35'!
resourceSelection
	^ self resources indexOf: selectedResource! !

!PackageResourcesList methodsFor: 'morphic' stamp: 'edc 1/26/2013 07:36'!
resourceSelection: aNumber
	selectedResource := self resources at: aNumber ifAbsent: [].
	self changed: #resourceSelection! !

!PackageResourcesList methodsFor: 'as yet unclassified' stamp: 'edc 1/26/2013 08:19'!
resources
|	list |
	list := OrderedCollection new.
	 self thisPackage resources keysAndValuesDo:[:k :v| list add: k  ].
	^list! !

!PackageResourcesList methodsFor: 'as yet unclassified' stamp: 'edc 11/23/2006 11:24'!
thisPackage
	"Answer the value of thisPackage"

	^ thisPackage! !

!PackageResourcesList methodsFor: 'as yet unclassified' stamp: 'edc 11/23/2006 11:24'!
thisPackage: anObject
	"Set the value of thisPackage"

	thisPackage := anObject! !

!PackageResourcesList methodsFor: 'actions' stamp: 'edc 1/26/2013 07:16'!
update: aSymbol
	aSymbol = #resources ifTrue:
		[resources := nil.
		self changed: #resources; changed: #resourceSelection]! !

Object subclass: #PackageServices
	instanceVariableNames: ''
	classVariableNames: 'ServiceClasses'
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 13:01'!
allServices
	^ ServiceClasses gather: [:ea | ea services]! !

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 12:59'!
initialize
	ServiceClasses := Set new! !

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 12:59'!
register: aClass
	ServiceClasses add: aClass! !

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 12:59'!
unregister: aClass
	ServiceClasses remove: aClass! !

!PositionableStream methodsFor: '*packageinfo-base' stamp: 'ul 6/17/2011 12:28'!
untilEnd: aBlock displayingProgress: aString
	aString
		displayProgressFrom: 0 to: self size
		during:
			[:bar |
			[self atEnd] whileFalse:
				[bar value: self position.
				aBlock value]].! !

!String methodsFor: '*packageinfo-base' stamp: 'nk 8/30/2004 09:02'!
escapeEntities
	^ self species streamContents: [:s | self do: [:c | s nextPutAll: c escapeEntities]]
! !
PackageInfo initialize!
PackageList initialize!
PackageServices initialize!
